# spatialIO - An integrated cloud-ready geospatial data management system of the Helmholtz Center for Environmental Research (UFZ)

A Spatial Data Infrastructure (SDI) is a combination of policies, standards and software to manage and deliver geospatial data ([Simmons, 2018](https://doi.org/10.1016/B978-0-12-409548-9.09611-1)). 
A good SDI follows policies and standards that are (widely) accepted in the communities (e.g.[FAIR](https://www.go-fair.org/fair-principles/), [OGC](https://www.ogc.org/)). 
Although often providing new functionality, the main advantage of an SDI is the connection of different tools and software products to build (mostly) automated workflows. 
This allows for less manual processing (and therefore fewer errors) as well as standardised data products due to fixed workflows. For this to work flawlessly, extensive documentation and user instructions are key. 
An SDI can contain (but is not limited to) data storage, metadata catalogue, tools for data processing, WebGIS and a form of data access (e.g. download, web service).

Climate modelling and research increasingly produce and share the data standard of [netCDF](https://www.unidata.ucar.edu/software/netcdf/) files, leading to an increasing demand in automated management of netCDF data. 
This application aims to provide automated workflows to manage standardized netCDF data and display them in an interactive WebGIS. The standard specifications follow the [Binding Regulations for Storing Data as netCDF Files](https://hereon-netcdf.readthedocs.io/en/latest/). 
Other vector and raster data formats ((Cloud optimized) GeoTIFF, sensor data) can be included in the workflows with manual work-steps and will be automated in the next versions. 
The application will be expanded continuously into a self-service platform to create custom WebGIS, automated workflows and various (meta-)data provision interfaces for a wide range of spatial data formats.

## Requirements

- Simply FAIR
- Science- and Management friendly: Provide interoperable and reliable netCDF data enriched by metadata and with provenance information.
- User friendly: Easy to use user interface for people that manage netCDf data or create WebGIS for netCDF data, without requiring knowledge about underlying technologies like databases.
- Admin friendly: A scalable and transferable container based solution that will smoothly integrate into typical scientific IT landscapes.
- Developer friendly: Common open source solutions structured by microservice architecture to keep it open and simple to extend for developers.

## Features

- S3 cloud-storage with [MinIO](https://min.io/)
- [FROST®-Server](https://www.iosb.fraunhofer.de/de/projekte-produkte/frostserver.html) to store and access sensor data
- Creation of custom interactive WebGIS components for netCDF, STA and GeoTIFF data
- Extendable processes to get spatially aggregated values for netCDF and GeoTIFF data
- Use of django framework to make configuration of data and WebGIS user-friendly
- Workflow for automated creation of OGC web services with [GeoServer](https://geoserver.org/) of new netCDF data
- Workflow for automated creation of metadata entries in [GeoNetwork](https://geonetwork-opensource.org/)
- [THREDDS Data Server (TDS)](https://www.unidata.ucar.edu/software/tds/) to provide netCDF data with [OPeNDAP](https://www.opendap.org/)

## Licence

All software and components written within the timeIO project is currently licensed under the [HEESIL](https://git.gfz-potsdam.de/rse/heesil).

## Getting Started

#### 1. Setup Django Admin-Frontend (GDI-Admin)

Build the backend container:


    docker-compose build backend


Start Backend by executing:


    docker-compose up -d backend

Get into the container:

  
    docker-compose exec backend bash

Create all tables in the database:


    python manage.py migrate

Load UFZ Django Theme (GDI-Admin) and initial data:


    python manage.py loaddata ufz_theme

    python manage.py loaddata initial_data

Create a first user (e.g. admin/admin):

    
    python manage.py createsuperuser

Go to [http://localhost:5001/gdi-backend/admin/](http://localhost:5001/gdi-backend/admin/) and log in


#### 2. Minio

###### step 1 - bucket

Start now all other containers including Minio

    docker-compose up -d


- go to [http://localhost:9001/login](http://localhost:9001/login) and log in (minioadmin/minioadmin)
- create a bucket, e.g. **'wis-d'**


###### step 2 - configure notifications
- run minio-client


    ./bin/run_mc.sh

    mc alias set minio-test http://minio:9000 minioadmin minioadmin

    mc event add minio-test/wis-d arn:minio:sqs::PRIMARY:postgresql --event put,delete


> The activity in the bucket **wis-d** will now produce notifications in the postgres database


#### 3. GeoNetwork 

you could see records created by the metadata-worker by going to: 

- go to [http://localhost:8081/geonetwork/](http://localhost:8081/geonetwork/)
- login with admin/admin

- create a group for the minio-bucket, e.g. **'wis-d'**
- look up the ID for this group in the geonetwork api (openapi interface under 'groups' > 'try out')
- create/update the bucket-entry in the Django-Backend with this ID
- now the metadata-worker is able to create records in GeoNetwork

#### 4. Worker 

to start the worker run: 


    docker compose -f docker-compose.yml -f docker-compose-worker.yml up datastore-worker
    docker compose -f docker-compose.yml -f docker-compose-worker.yml up metadata-worker



> View the logs of running container by executing. e.g.: 'docker-compose logs -f metadata-worker'


## Production-Build Backend

run: 

    
    docker compose --env-file .env-prod -f docker-compose-prod.yml up -d backend

> to create a new image, you could use [a useful script](build-backend-image.sh)



## save current configuration of production database
- start backend with production credentials
- run:


    python manage.py dumpdata --exclude=auth --exclude=admin --exclude=contenttypes --exclude=sessions --exclude=main.processcache --exclude=main.bucketevent -o main/fixtures/prod_db_content.json

- import the data:


    python manage.py loaddata prod_db_content


## Techstack, dependencies and third party open source products

- [MinIO](https://min.io/)
- [FROST®-Server](https://www.iosb.fraunhofer.de/de/projekte-produkte/frostserver.html)
- [GeoServer](https://geoserver.org/)
- [GeoNetwork](https://geonetwork-opensource.org/)
- [THREDDS Data Server (TDS)](https://www.unidata.ucar.edu/software/tds/)
- [django](https://www.djangoproject.com/)
- [PostgreSQL](https://www.postgresql.org/)
- [Vue](https://vuejs.org/)
- [Bulma](https://bulma.io/)
- [pygeoapi](https://pygeoapi.io/)

#### GeoServer

- Python library for using the api: https://pypi.org/project/geoserver-restconfig/

#### Django
- documentation: https://docs.djangoproject.com/en/4.0/
- alternate theme: https://github.com/fabiocaccamo/django-admin-interface
- custom commands: https://docs.djangoproject.com/en/4.0/howto/custom-management-commands/
- ColorField: https://pypi.org/project/django-colorfield/

#### S3 mount tutorial
- https://blog.meain.io/2020/mounting-s3-bucket-kube/
- python s3fs: https://s3fs.readthedocs.io/en/latest/api.html#s3fs.core.S3FileSystem.read_block

#### Minio Notifications
- https://docs.min.io/minio/baremetal/monitoring/bucket-notifications/publish-events-to-postgresql.html
- [example-notification](app/main/lib/utils/README.md)

## Contact

Feel free to contact [Christian Schulz](mailto:christian.schulz@ufz.de) and [Rebekka Lange](mailto:rebekka.lange@ufz.de) or the [UFZ RDM team](https://www.ufz.de/index.php?en=45348) in general with every question you may have.

## Contributors

- Christian Schulz (UFZ)
- Rebekka Lange (UFZ)
- Hannes Bohring (UFZ)
- Norman Ziegner (UFZ)
- Jan Bumberger (UFZ)

[Imprint](https://www.ufz.de/index.php?en=36683)