from django.contrib import admin

from main.models import Area, AreaLayer, DataVariable, Bucket, Project, WmsLayer, WmsLayerProcess, WmsLayerLegend, WmsLegend, ConnectedLayer
from main.forms.layer.area_layer import AreaFilter, AreaLayerInline

from main.lib.utils.backend import get_user_groups

from django.db import models
from django.forms import Textarea

from main.lib.utils.worker_utils import get_geoserver_api


class DataVariableFilter(admin.SimpleListFilter):
    title = 'Variable'
    parameter_name = 'variable'

    def lookups(self, request, model_admin):
        result = []
        if request.user.is_superuser:
            variables = DataVariable.objects.all()
        else:
            variables = DataVariable.objects.filter(group__in=get_user_groups(request))

        for variable in variables:
            result.append((variable.id, variable.name))

        result.append((0, 'None'))

        return result

    def queryset(self, request, query_set):

        if self.value():
            if int(self.value()) == 0:
                return query_set.filter(variable__isnull=True)

            return query_set.filter(variable=self.value())
        else:
            return query_set


class WmsLayerLegendInline(admin.TabularInline):
    model = WmsLayerLegend
    min_num = 0
    extra = 0

    def formfield_for_foreignkey(self, db_field, request, **kwargs):

        if not request.user.is_superuser:
            # if not superuser, a user can only select legends from his groups
            if db_field.name == "legend":
                kwargs["queryset"] = WmsLegend.objects.filter(group__in=get_user_groups(request))

        return super().formfield_for_foreignkey(db_field, request, **kwargs)


class LayerProcessInline(admin.TabularInline):
    model = WmsLayerProcess
    min_num = 0
    extra = 0

    def get_readonly_fields(self, request, obj):
        if request.user.is_superuser:
            return []
        return ['in_cache']


class ConnectedLayerInline(admin.TabularInline):
    model = ConnectedLayer
    fk_name = 'wms_layer'
    min_num = 0
    extra = 0

    def formfield_for_foreignkey(self, db_field, request, **kwargs):

        if not request.user.is_superuser:
            # if not superuser, a user can only select layers from his buckets
            if db_field.name == "referenced_wms_layer":
                kwargs["queryset"] = WmsLayer.objects.filter(file_record__bucket__group__in=get_user_groups(request))

        return super().formfield_for_foreignkey(db_field, request, **kwargs)


class WmsLayerAdmin(admin.ModelAdmin):
    model = WmsLayer
    inlines = [AreaLayerInline, WmsLayerLegendInline, ConnectedLayerInline, LayerProcessInline ]
    list_display = ('__str__', 'variable', 'get_file_path', 'get_position', 'get_is_active', 'get_is_default')
    list_filter = (AreaFilter, DataVariableFilter,)
    save_as = True

    search_fields = ["layer_name", "name_de", "name_en", "info_de", "info_en", "wms_url"]

    formfield_overrides = {
        models.TextField: {'widget': Textarea(
            attrs={'rows': 15,
                   'cols': 60})},
    }

    fieldsets = [
        (None, {
            'fields': ('layer_name', ('name_de', 'name_en'), ),
        }),
        ('optional', {
            'fields': (('unit_de', 'unit_en'), ('info_de', 'info_en'), ('y_axis_min', 'y_axis_max'), ('time_format', 'scale_factor', 'opacity', 'is_point_request_enabled'), 'uncertainty_path',),
            'classes': ('collapse',),
        }),
        ('Dataset', {
            'fields': (('file_record'), ('no_data_value', 'epsg_code'), ('variable', 'axis_label_x', 'axis_label_y'), 'time_steps', 'wms_url'),
            'classes': ('collapse',),
        })
    ]

    @admin.display(description='is active')
    def get_is_active(self, obj):
        area_layer = AreaLayer.objects.get(wms_layer=obj)
        if area_layer:
            return area_layer.is_active
        return False

    @admin.display(description='is default')
    def get_is_default(self, obj):
        area_layer = AreaLayer.objects.get(wms_layer=obj)
        if area_layer:
            return area_layer.is_default_on_start
        return False

    @admin.display(description='position')
    def get_position(self, obj):
        area_layer = AreaLayer.objects.get(wms_layer=obj)
        if area_layer:
            return area_layer.position
        return False

    @admin.display(description='file path')
    def get_file_path(self, obj):
        result = obj.file_record.file_path
        length = len(result)
        if length > 30:
            result = "..." + result[length - 30:]
        return result

    def get_readonly_fields(self, request, obj):
        fields = ['wms_url', 'layer_name', 'variable', 'axis_label_x', 'axis_label_y', 'time_steps']
        if request.user.is_superuser:
            return []
        return fields

    def get_queryset(self, request):
        # if not superuser, a user can see only see layers from his buckets/groups
        qs = super().get_queryset(request)

        buckets = Bucket.objects.filter(group__in=get_user_groups(request))
        if request.user.is_superuser:
            return qs
        return qs.filter(file_record__bucket__in=buckets)

    def save_related(self, request, form, formset, change):
        super(WmsLayerAdmin, self).save_related(request, form, formset, change)

        wms_layer = form.instance
        try:
            layer_legends = WmsLayerLegend.objects.filter(wms_layer=wms_layer).order_by('position')
            geoserver_api = get_geoserver_api()
            for layer_legend in layer_legends:
                if layer_legend == layer_legends[0]:
                    geoserver_api.add_style(layer_legends[0].legend.name_de, wms_layer.name_de, True)
                    geoserver_api.add_style(layer_legends[0].legend.name_en, wms_layer.name_en, True)
                else:
                    geoserver_api.add_style(layer_legend.legend.name_de, wms_layer.name_de)
                    geoserver_api.add_style(layer_legend.legend.name_en, wms_layer.name_en)
        except Exception as e:
            print('{}'.format(str(e)))
