from django.contrib import admin
from django.contrib.auth.models import Group

from main.models import Area, ListFilter, ListFilterItem, AreaLayer, LayerListFilterItem
from main.lib.utils.backend import get_user_groups

import nested_admin


class LayerListFilterItemInline(nested_admin.NestedStackedInline):
    model = LayerListFilterItem
    min_num = 0
    extra = 0
    classes = ("collapse", "nested_inline_form",)

    def formfield_for_foreignkey(self, db_field, request, **kwargs):

        if db_field.name == "layer":

            if 'object_id' in request.resolver_match.kwargs:
                filter_id = request.resolver_match.kwargs['object_id']
                list_filter = ListFilter.objects.get(id=filter_id)
                if list_filter:
                    kwargs["queryset"] = AreaLayer.objects.filter(area=list_filter.area, is_active=True).order_by('position')
            else:
                # if no Area is assigned, no Arealayers should be available.
                # A user should only be able to select Layers of one specific area in all cases
                kwargs["queryset"] = AreaLayer.objects.none()

        return super().formfield_for_foreignkey(db_field, request, **kwargs)


class ListFilterItemInline(nested_admin.NestedStackedInline):
    model = ListFilterItem
    inlines = [LayerListFilterItemInline]
    min_num = 0
    extra = 0

    fieldsets = [
        (None, {
            'fields': (('name_de', 'name_en'), 'list_filter'),
        }),
    ]

    def get_queryset(self, request):
        # if not superuser, a user can see only see ListFilter from his areas/projects
        qs = super().get_queryset(request)

        if request.user.is_superuser:
            return qs
        return qs.filter(list_filter__area__project__bucket__group__in=get_user_groups(request))


class ListFilterAdmin(nested_admin.NestedModelAdmin):
    inlines = [ListFilterItemInline]
    list_display = ('__str__', 'area', 'position', 'is_active')

    fieldsets = [
        (None, {
            'fields': (('name_de', 'name_en'), ),
        }),
        (None, {
            'fields': (('area', 'position'), 'is_active', ),
            'description': "(When creating a new ListFilter, click on 'Save and continue editing' to reduce the layers below to the selected area.)"
        }),
    ]

    def formfield_for_foreignkey(self, db_field, request, **kwargs):

        if db_field.name == "area":

            # if not superuser, a user can only select areas from his projects
            if not request.user.is_superuser:
                kwargs["queryset"] = Area.objects.filter(project__bucket__group__in=get_user_groups(request))

        return super().formfield_for_foreignkey(db_field, request, **kwargs)

    def get_queryset(self, request):
        # if not superuser, a user can see only see ListFilter from his areas/projects
        qs = super().get_queryset(request)

        if request.user.is_superuser:
            return qs
        return qs.filter(area__project__bucket__group__in=get_user_groups(request))
