from django.contrib import admin
from django import forms
from django.core.exceptions import ValidationError


from django.db import models
from django.forms import Textarea

from main.models import Area, Bucket, Project
from main.lib.utils.backend import get_user_groups


class AreaInlineFormset(forms.BaseInlineFormSet):
    model = Area

    def clean(self):
        count_default_areas = 0
        for form in self.forms:
            cleaned_data = form.clean()

            if cleaned_data == {}:
                continue    # form is empty and will not be saved. Skip validation.

            if not cleaned_data.get('DELETE', False):
                if cleaned_data['is_default_on_start']:
                    count_default_areas += 1

            if count_default_areas > 1:
                raise ValidationError("A project can only have one default area.")


class AreaInline(admin.StackedInline):
    model = Area
    formset = AreaInlineFormset
    min_num = 1
    extra = 0

    fieldsets = [
        (None, {
            'fields': ('position', ),
        }),
        (None, {
            'fields': (('name_de', 'name_en'), ),
        }),
        (None, {
            'fields': (('is_active', 'is_default_on_start'), ),
        }),
        (None, {
            'fields': (('info_de', 'info_en'), ),
        }),
    ]


class ProjectAdmin(admin.ModelAdmin):
    model = Project
    inlines = [AreaInline]

    formfield_overrides = {
        models.TextField: {'widget': Textarea(
            attrs={'rows': 15,
                   'cols': 60})},
    }
    fieldsets = [
        (None, {
         'fields': ('bucket', ('name_de', 'name_en'), 'is_english_version_active'),
        }),
        ('Description', {
            'fields': (('teaser_de', 'teaser_en'), ('about_text_de', 'about_text_en'),('imprint_text_de', 'imprint_text_en'), ('faq_text_de', 'faq_text_en')),
            'classes': ('collapse',),
        }),
        ('Technical', {
            'fields': (('default_lon', 'default_lat',  'default_zoom'),),
        }),
        ('Gallery', {
            'fields': (('show_in_gallery', 'gallery_image',  'position_in_gallery'),),
        }),
        ('Admin', {
            'fields': (('name_in_url', 'name_in_backend'), ('maintenance_notification_de', 'maintenance_notification_en'), 'theme',),
            'classes': ('collapse',),
        }),
    ]

    def get_readonly_fields(self, request, obj):
        fields = ['bucket', 'name_in_url', 'name_in_backend', 'maintenance_notification_de', 'maintenance_notification_en', 'theme', 'position_in_gallery']
        if request.user.is_superuser:
            return []
        return fields

    def get_queryset(self, request):
        # if not superuser, a user can see only projects from his groups
        qs = super().get_queryset(request)

        buckets = Bucket.objects.filter(group__in=get_user_groups(request))
        if request.user.is_superuser:
            return qs
        return qs.filter(bucket__in=buckets)
