import requests
import json

from xml.etree import ElementTree
from geoserver.catalog import Catalog

geotiff_extension = ".tif"

GEOSERVER_MOSAIC_DIR = "/opt/geoserver/data_dir/mosaic"


class GeoServerApi:

    def __init__(self, url, username, password):
        self.cat = Catalog(
            url,
            username,
            password
        )
        self.url = url
        self.username = username
        self.password = password

        self.coverages = []

        self.job_status = 0
        self.logs = []

    def log(self, log):
        self.logs.append(str(log))

    def set_error_status(self):
        self.job_status = 1

    def reset(self):
        self.job_status = 0
        self.logs = []

    def add_granule_to_mosaic(self, workspace, coverage_store, coverage, absolute_path, filename):

        stores = self.cat.get_stores([coverage_store])

        if len(stores) == 1:
            store = stores[0]

            self.cat.add_granule(absolute_path, store, workspace)

            granules = self.cat.list_granules(coverage, store, workspace)
            granule_id = self.find_granule_id_by_filename(filename, granules)

            print("Granule added to {}: {}".format(coverage_store, filename))
            return granule_id
        else:
            print("Coveragestore could not be found: {}".format(coverage_store))

    def create_workspace(self, workspace_name):
        # check for existing workspace
        workspace = self.cat.get_workspace(workspace_name)

        if workspace is None:
            try:
                self.cat.create_workspace(workspace_name, workspace_name)
                self.log("Workspace created: {}".format(workspace_name))
            except Exception as e:
                self.log("Workspace could not be created: {}".format(e))
                self.set_error_status()
        else:
            self.log("Workspace already exists. Existing Workspace will be used: {}".format(workspace_name))

    def create_netcdf_datastore(self, workspace, coverage_store, absolute_path):

        self.create_workspace(workspace)

        headers = {"Content-type": "text/plain"}
        url = '{0}workspaces/{1}/coveragestores/{2}/external.netcdf'.format(
            self.url, workspace, coverage_store)

        self.log(url)
        self.log(absolute_path)

        try:
            response = requests.put(url, data=absolute_path, auth=(self.username, self.password), headers=headers)
            if response.status_code != 201:
                self.log("Create status not ok:")
                self.log(response.content)
                self.set_error_status()
            else:
                self.log("Coveragestore {} created successfully".format(coverage_store))
                # now enable time dimension if possible
                self.add_time_to_datastore(workspace, coverage_store)
        except Exception as e:
            print("Coveragestore could not be created: {}".format(e))
            self.set_error_status()

    def create_mosaic_datastore(self, name, workspace):
        self.cat.create_imagemosaic(
            name=name,
            data='file://{}/{}'.format(GEOSERVER_MOSAIC_DIR, name),
            workspace=workspace,
            overwrite=True
        )

        self.add_time_to_datastore(workspace, name)

    def create_geotiff_datastore(self, name, workspace, path):
        if self.cat.get_stores([name]):
            print("Geotiff-Datastore already exists.")
            return

        self.cat.create_coveragestore(
            name,
            workspace,
            path,
            layer_name=name,
            type='GeoTIFF',
            create_layer=True
        )

    def _create_new_style(self, style_name, style_xml):
        try:
            headers = {"Content-type": "application/vnd.ogc.sld+xml"}
            url = '{0}styles?raw=true'.format(self.url)
            response = requests.post(url=url, auth=(self.username, self.password), headers=headers, data=style_xml)
            if response.status_code != 201:
                print('Style could not be created: {}'.format(response.content))
            else:
                print('Style created successfully')
        except Exception as e:
            print('Style could not be filled/updated: {}'.format(str(e)))

    def _update_style(self, style_name, style_xml):
        try:
            headers = {"Content-type": "application/vnd.ogc.sld+xml"}
            url = '{0}styles/{1}?raw=true'.format(self.url, style_name)
            response = requests.put(url=url, auth=(self.username, self.password), headers=headers, data=style_xml)
            if response.status_code != 200:
                print('Style could not be updated: {}'.format(response.content))
            else:
                print('Style updated successfully')
        except Exception as e:
            print('Style could not be filled/updated: {}'.format(str(e)))

    def create_style(self, style_name, style_xml):
        try:
            url = '{0}styles/{1}.sld'.format(self.url, style_name)
            response = requests.get(url=url, auth=(self.username, self.password))

            style_xml = style_xml.encode('utf-8')

            if response.status_code != 200:
                print('Style does not exist yet. Style will be added as new.')
                self._create_new_style(style_name, style_xml)
            else:
                print('Update existing style')
                self._update_style(style_name, style_xml)
        except Exception as e:
            print('Could not retrieve style: {}'.format(str(e)))

    def add_style(self, style_name: str, layer_name: str, default: bool = False):

        headers = {"content-type": "application/json"}
        url = '{0}layers/{1}/styles?default={2}'.format(self.url, layer_name, default)

        body = {
            "style": {
                "name": style_name,
                "filename": style_name + '.sld'
            }
        }

        try:
            response = requests.post(url, data=json.dumps(body), auth=(self.username, self.password), headers=headers)

            if response.status_code != 201:
                print("Create status not ok: {}".format(response.content))
                self.set_error_status()
            else:
                print("Style linked successfully")

        except Exception as e:
            print("Style {0} could not be linked to layer {1}: {2}".format(style_name, layer_name, e))
            self.set_error_status()

    def add_time_to_datastore(self, workspace, coverage_store):
        # to enable time dimension to a coverage, we first need the native names of them
        coverages = self.get_coverages(workspace, coverage_store)

        self.coverages = coverages

        for coverage in coverages:
            url = "{0}workspaces/{1}/coveragestores/{2}/coverages/{3}".format(
                self.url, workspace, coverage_store, coverage)
            headers = {"content-type": "application/xml"}
            time_data = (
                "<coverage>"
                "<enabled>true</enabled>"
                "<metadata>"
                "<entry key='time'>"
                "<dimensionInfo>"
                "<enabled>true</enabled>"
                "<presentation>LIST</presentation>"
                "<units>ISO8601</units>"
                "<defaultValue>"
                "<strategy>MAXIMUM</strategy>"
                "</defaultValue>"
                "</dimensionInfo>"
                "</entry>"
                "</metadata>"
                "</coverage>"
            )
            try:
                response = requests.put(url, data=time_data, auth=(self.username, self.password), headers=headers)
                if response.status_code not in [200, 201]:
                    self.log("Time dimension status for coverage {} not ok: {}".format(coverage, response.content))
                    self.set_error_status()
                else:
                    self.log("Time dimension for coverage {} enabled successfully".format(coverage))
            except Exception as e:
                self.log("Time dimension for coverage {} could not be enabled: {}".format(coverage, e))
                self.set_error_status()

    def get_coverages(self, workspace, coverage_store):
        url = "{0}workspaces/{1}/coveragestores/{2}/coverages".format(self.url, workspace, coverage_store)
        coverages = []
        try:
            response = requests.get(url, auth=(self.username, self.password))
            if response.status_code != 200:
                self.log("Get coverage names status not ok: {}".format(response.content))
                self.set_error_status()
            else:
                xml = ElementTree.fromstring(response.text)
                for child in xml:
                    coverages.append(child.find("name").text)
                self.log("Coverage names retrieved successfully: {}".format(coverages))
        except Exception as e:
            self.log("Coverage names could not be retrieved: {}".format(e))
            self.set_error_status()
        return coverages

    def remove_granule(self, coverage, coverage_store, granule_id, workspace):
        self.cat.delete_granule(coverage, coverage_store, granule_id, workspace)
        print("Granule removed from {}: {}".format(coverage_store, granule_id))

    @staticmethod
    def find_granule_id_by_filename(filename, granules):
        filename = filename.replace(geotiff_extension, '')
        for granule in granules['features']:
            if granule['properties']['file_name'] == filename:
                return granule['id']

        raise Exception("Filename not found in granules!")
