import os

from django.core.management.base import BaseCommand

from owslib.wms import WebMapService

from main.models import Bucket, BucketEvent, FileRecord, WmsLayer, DataVariable
from main.lib.utils.worker_utils import *
from main.lib.utils.bucketevent_utils import *
from main.lib.utils.backend import create_file_indentifier
from main.lib.utils.s3_utils import download_minio_file


wms_url = os.environ.get("GEOSERVER_URL") + '/ows?service=wms&version=1.3.0&request=GetCapabilities'


class Command(BaseCommand):

    def handle(self, *args, **options):

        geoserver_api = get_geoserver_api()

        while True:
            events = BucketEvent.objects.filter(is_new__isnull=True, has_metadata_record__isnull=False)

            for event in events:

                bucket_name = get_bucket_name(event)
                relative_path = get_relative_path(event)

                coverage_store = extract_coverage_name(extract_name_for_dataset(event))

                if is_netcdf(event):
                    # layers need here a unique name, the pattern in GeoServer is workspace:coverage
                    workspace = bucket_name + "_" + create_name_for_netcdf_workspace(extract_name_for_dataset(event))

                    if is_create_event(event):

                        bucket = None

                        try:
                            bucket = Bucket.objects.get(name=bucket_name)
                        except Bucket.DoesNotExist:
                            print('Bucket is not configured: ' + bucket_name)

                        if bucket:

                            # if event is in Bucket for viewer
                            if bucket.connect_to_geoserver:

                                absolute_path = '/{}/{}/{}'.format(get_data_dir(), bucket_name, relative_path)

                                is_file_copied = download_minio_file(bucket_name, relative_path, absolute_path)

                                if is_file_copied:

                                    geoserver_api.create_netcdf_datastore(workspace, coverage_store, absolute_path)

                                    file_path = '{}/{}'.format(bucket_name, relative_path)

                                    try:
                                        file_record = FileRecord.objects.get(file_path=file_path)

                                    except FileRecord.DoesNotExist:
                                        print("file_record does not exist: " + file_path)
                                        continue

                                    layers = WmsLayer.objects.filter(file_record=file_record)

                                    if len(layers) == 0:

                                        backend_proxy_url = os.environ.get("BACKEND_URL") + '/proxy/' + workspace + '/wms'

                                        new_layer = WmsLayer(
                                            file_record=file_record,
                                            wms_url=backend_proxy_url,
                                            epsg_code='EPSG:4326',
                                            axis_label_x='Long',
                                            axis_label_y='Lat',
                                            scale_factor=1
                                        )

                                        _update_by_wms_and_save(new_layer, workspace)

                                    else:
                                        for layer in layers:
                                            _update_by_wms_and_save(layer, workspace)

                                else:
                                    print('Could not download file: ' + relative_path)

                            if bucket.connect_to_thredds:

                                absolute_path = '/{}/thredds/{}/{}'.format(get_data_dir(), bucket_name, relative_path)

                                is_file_copied = download_minio_file(bucket_name, relative_path, absolute_path)

                                if is_file_copied:
                                    print('Downloaded to Thredds successful: ' + relative_path)
                                else:
                                    print('Could not download file to Thredds: ' + relative_path)

                    if is_delete_event(event):
                        if bucket:
                            if bucket.connect_to_thredds:
                                absolute_path = '/{}/thredds/{}/{}'.format(get_data_dir(), bucket_name, relative_path)
                                try:
                                    os.remove(absolute_path)
                                except OSError:
                                    pass

                if geoserver_api.job_status == 1:
                    event.job_status = 1
                else:
                    event.job_status = 0

                logs = geoserver_api.logs
                event.logs = logs
                # set the event as processed
                event.is_new = False
                geoserver_api.reset()

                event.save()

            sleep_by_defined_duration()


def _update_by_wms_and_save(layer, workspace):

    try:
        wms = WebMapService(wms_url, version='1.3.0')

        # iterate all available wms 'LAYERS' which exist in Geoserver
        for wms_variable in list(wms.contents):

            # now iterate all data-variables which are defined in GDI-Admin as Variable for Layers
            for data_variable in DataVariable.objects.all():

                complete_name = workspace + ':' + data_variable.name

                if complete_name == wms_variable:

                    layer.variable = data_variable
                    layer.layer_name = complete_name

                    time_steps = wms.contents[complete_name].timepositions

                    if time_steps is None:
                        layer.time_steps = []
                    else:
                        layer.time_steps = time_steps
                    break
        layer.save()

    except Exception as e:
        print("Could not update layer {}: {}".format(layer.id, str(e)))
