import os
import sys
import s3fs
import json
import time

from main.lib.metadata.MetadataService import MetadataService
from main.lib.metadata.NetcdfParser import NetcdfParser
from main.lib.metadata.JsonParser import JsonParser

from main.lib.utils.worker_utils import *
from main.lib.utils.bucketevent_utils import *
from main.lib.utils.s3_utils import get_s3_filesystem
from main.models import Bucket, BucketEvent, FileRecord, Layer
from main.lib.utils.backend import create_file_indentifier

geonetwork_api = get_geonetwork_api()

##########################################################################################################
# this worker creates records in geonetwork for netcdf-datastores in GeoServer
# metadata gets extracted from netcdf-header and a download-link is provided by Minio-Client
##########################################################################################################

fs = get_s3_filesystem()


def create_file_record(bucket, file_path, event, parser, data_files):

    try:
        file_record = FileRecord.objects.get(bucket=bucket, file_path=file_path)
        file_record.data_files = data_files
        print('file_record already created')
    except FileRecord.DoesNotExist:
        print('cannot find record: ' + file_path)
        file_record = FileRecord(
            bucket=bucket,
            file_path=file_path,
            data_files=data_files,
            file_identifier=create_file_indentifier()
        )

    file_identifier = file_record.file_identifier

    is_geonetwork_created = False

    parser.parse(file_path)

    metadata_service = MetadataService(parser)

    try:
        if file_record.geonetwork_uuid:
            xml = metadata_service.create_xml(bucket_name, get_object_key(event), file_identifier, file_record.geonetwork_uuid)
            geonetwork_api.update_record(xml, bucket.geonetwork_group)
            geonetwork_api.log("Geonetwork-Record updated.")

        else:
            xml = metadata_service.create_xml(bucket_name, get_object_key(event), file_identifier)
            uuid = geonetwork_api.create_record(xml, bucket.geonetwork_group)

            if uuid:
                file_record.geonetwork_uuid = uuid
            geonetwork_api.log("Geonetwork-Record created.")

        is_geonetwork_created = True
    except:
        geonetwork_api.log("Error in creating Geonetwork-Record.")

    file_record.save()
    return is_geonetwork_created


def create_geojson_file_record(bucket, file_path):

    try:
        file_record = FileRecord.objects.get(bucket=bucket, file_path=file_path)
        print('file_record already created')
    except FileRecord.DoesNotExist:
        print('cannot find record: ' + file_path)
        file_record = FileRecord(
            bucket=bucket,
            file_path=file_path,
            file_identifier=create_file_indentifier()
        )

    file_identifier = file_record.file_identifier

    is_geonetwork_created = True

    file_record.save()
    return is_geonetwork_created


def delete_file_record(bucket, file_path):
    try:
        file_record = FileRecord.objects.get(bucket=bucket, file_path=file_path)

        geonetwork_api.delete_record(file_record.geonetwork_uuid)

        print("Delete file_record with UUID: {}".format(file_record.geonetwork_uuid))
        file_record.delete()

    except (Bucket.DoesNotExist, FileRecord.DoesNotExist):
        print('could not find file_record to delete: ' + file_path)


def get_data_file_prefixes(metadata_file):
    try:
        with fs.open(metadata_file, 'rb') as json_file:
            file = json.load(json_file)
            return file['data']
    except Exception as e:
        print('JSON file could not be read: ' + str(e))
        return


while True:
    events = BucketEvent.objects.filter(has_metadata_record__isnull=True)

    # noinspection PyTypeChecker
    for event in events:

        has_metadata_record = False

        bucket_name = get_bucket_name(event)
        relative_path = get_relative_path(event)

        try:
            bucket = Bucket.objects.get(name=bucket_name)
        except Bucket.DoesNotExist:
            print('bucket does not exist: ' + bucket_name)
            continue

        file_path = '{}/{}'.format(bucket_name, relative_path)

        if is_create_event(event):

            if is_netcdf(event):
                parser = NetcdfParser(fs)
                has_metadata_record = create_file_record(bucket, file_path, event, parser, None)

            elif file_path.endswith('.geojson'):
                has_metadata_record = create_geojson_file_record(bucket, file_path)

            elif file_path.endswith('.json'):
                parser = JsonParser(fs)
                has_metadata_record = create_file_record(bucket, file_path, event, parser, get_data_file_prefixes(file_path))

        if is_delete_event(event):
            delete_file_record(bucket, file_path)

        event.has_metadata_record = has_metadata_record
        event.logs = geonetwork_api.logs
        geonetwork_api.reset()

        event.save()

    sleep_by_defined_duration()
