from django.db import models
from django.conf import settings
from django.contrib.postgres.fields import ArrayField
from django.contrib.auth.models import Group
from colorfield.fields import ColorField
import uuid

from django.core.exceptions import ValidationError

TIME_FORMATS = (
    ('month', 'month'),
    ('period', 'period'),
    ('season', 'season'),
    ('year', 'year'),
)

LEGEND_TYPES = (
    ('discrete', 'discrete'),
    ('gradient', 'gradient'),
)

AGGREGATION = 'aggregation'
PROCESS = 'process'
MAGNITUDE = 'magnitude'
ANOMALY = 'anomaly'

COMPUTATION_TYPES = (
    (AGGREGATION, AGGREGATION),
    (PROCESS, PROCESS),
    (MAGNITUDE, MAGNITUDE),
    (ANOMALY, ANOMALY)
)

REQUEST_TYPE_1 = 'timeseries'
REQUEST_TYPE_2 = 'single value'

REQUEST_TYPES = (
    (REQUEST_TYPE_1, REQUEST_TYPE_1),
    (REQUEST_TYPE_2, REQUEST_TYPE_2),
)


# abstract class giving german or english version of text
class CommonInfo(models.Model):
    name_de = models.CharField('Name (de)', max_length=1000, null=True)
    name_en = models.CharField('Name (en)', max_length=1000, null=True)

    class Meta:
        abstract = True

    def __str__(self):
        return self.name_de


class Bucket(models.Model):
    group = models.ForeignKey(Group, on_delete=models.CASCADE)
    name = models.CharField(max_length=100, unique=True)
    geonetwork_group = models.IntegerField(blank=True, null=True)
    connect_to_geoserver = models.BooleanField(default=False)
    connect_to_thredds = models.BooleanField(default=False)

    def __str__(self):
        return self.name


class Project(CommonInfo):
    bucket = models.ForeignKey(Bucket, on_delete=models.CASCADE)
    name_in_url = models.CharField(max_length=100, unique=True)
    name_in_backend = models.CharField(max_length=100, unique=True)
    teaser_de = models.TextField(blank=True, null=True)
    teaser_en = models.TextField(blank=True, null=True)
    about_text_de = models.TextField(blank=True, null=True)
    about_text_en = models.TextField(blank=True, null=True)
    imprint_text_de = models.TextField(blank=True, null=True)
    imprint_text_en = models.TextField(blank=True, null=True)
    faq_text_de = models.TextField(blank=True, null=True)
    faq_text_en = models.TextField(blank=True, null=True)
    theme = models.CharField(max_length=100, blank=True, null=True)
    default_lon = models.DecimalField('Longitude', max_digits=12, decimal_places=10, blank=True, null=True)
    default_lat = models.DecimalField('Latitude', max_digits=12, decimal_places=10, blank=True, null=True)
    default_zoom = models.DecimalField(max_digits=10, decimal_places=8, blank=True, null=True)
    maintenance_notification_de = models.CharField(max_length=4000, blank=True, null=True)  # hidden for user
    maintenance_notification_en = models.CharField(max_length=4000, blank=True, null=True)  # hidden for user
    is_english_version_active = models.BooleanField(default=False)
    show_in_gallery = models.BooleanField(default=False)
    position_in_gallery = models.IntegerField(blank=True, null=True)
    gallery_image = models.CharField(max_length=4000, blank=True, null=True)


class Area(CommonInfo):
    project = models.ForeignKey(Project, on_delete=models.CASCADE)
    position = models.IntegerField(default=1)
    is_active = models.BooleanField(default=True)
    info_de = models.TextField(blank=True, null=True)
    info_en = models.TextField(blank=True, null=True)
    is_default_on_start = models.BooleanField(default=False)

    class Meta:
        constraints = [
            models.UniqueConstraint(fields=['project', 'position'], name='unique_area_pos')
        ]


class WmsLegend(models.Model):
    name_de = models.CharField('Name (de)', max_length=50, unique=True)   # has to be unique for request to GeoServer
    name_en = models.CharField('Name (en)', max_length=50, unique=True)   # has to be unique for request to GeoServer
    group = models.ForeignKey(Group, on_delete=models.CASCADE)
    type = models.CharField(max_length=20, choices=LEGEND_TYPES, default='discrete')

    def __str__(self):
        return self.name_de


class WmsColor(CommonInfo):
    legend = models.ForeignKey(WmsLegend, on_delete=models.CASCADE)
    color = ColorField(format="hex")
    position = models.IntegerField(default=1)
    upper_limit = models.DecimalField(max_digits=12, decimal_places=6)
    is_invisible = models.BooleanField(default=False)

    class Meta:
        constraints = [
            models.UniqueConstraint(fields=['legend', 'position'], name='unique_color_pos'),
            models.UniqueConstraint(fields=['legend', 'upper_limit'], name='unique_color_limit')
        ]


class StaLegend(models.Model):
    name = models.CharField(max_length=1000)
    group = models.ForeignKey(Group, on_delete=models.CASCADE)

    def __str__(self):
        return self.name


class StaColor(models.Model):
    legend = models.ForeignKey(StaLegend, on_delete=models.CASCADE)
    color = ColorField(format="hex")
    position = models.IntegerField(default=1)
    upper_limit = models.DecimalField(max_digits=12, decimal_places=6)
    radius = models.IntegerField(default=7)

    def __str__(self):
        return "Color {} ({} {})".format(self.position, self.color, self.upper_limit)


class DataVariable(models.Model):
    name = models.CharField(max_length=1000)
    group = models.ForeignKey(Group, on_delete=models.CASCADE)

    def __str__(self):
        return self.name


class FileRecord(models.Model):
    bucket = models.ForeignKey(Bucket, on_delete=models.CASCADE)
    geonetwork_uuid = models.UUIDField(blank=True, null=True)
    file_path = models.CharField(max_length=4000, unique=True)  # is readonly
    data_files = ArrayField(models.CharField(max_length=4000), null=True, unique=True)
    file_identifier = models.CharField(max_length=32, blank=True, null=True, unique=True)

    def __str__(self):
        return self.file_path


class Layer(CommonInfo):
    info_de = models.TextField(blank=True, null=True)
    info_en = models.TextField(blank=True, null=True)
    time_format = models.CharField(max_length=20, choices=TIME_FORMATS, blank=True, null=True)
    y_axis_min = models.DecimalField(max_digits=12, decimal_places=5, blank=True, null=True)
    y_axis_max = models.DecimalField(max_digits=12, decimal_places=5, blank=True, null=True)

    class Meta:
        abstract = True


class StaEndpoint(models.Model):
    group = models.ForeignKey(Group, on_delete=models.CASCADE)
    base_url = models.CharField(max_length=4000)
    username = models.CharField(max_length=4000, blank=True, null=True)
    password = models.CharField(max_length=4000, blank=True, null=True)
    project = models.CharField(max_length=4000, blank=True, null=True)

    def __str__(self):
        if self.project:
            return "{} ({})".format(self.base_url, self.project)
        return self.base_url


# class StaThingFilter(models.Model):
#     endpoint = models.ForeignKey(StaEndpoint, on_delete=models.CASCADE)
#     thing_property_key = models.CharField(max_length=1000)
#     thing_property_value = models.CharField(max_length=1000)


class StaLayer(Layer):
    endpoint = models.ForeignKey(StaEndpoint, on_delete=models.CASCADE)
    observed_property_use_group = models.CharField(max_length=4000, blank=True, null=True)
    legend = models.ForeignKey(StaLegend, on_delete=models.CASCADE)
    geonetwork_uuid = models.UUIDField(blank=True, null=True)


# class StaObservedPropertyFilter(models.Model):
#     layer = models.ForeignKey(StaLayer, on_delete=models.CASCADE)
#     observed_property_property_key = models.CharField(max_length=1000)
#     observed_property_property_value = models.CharField(max_length=1000)


class WmsLayer(Layer):
    file_record = models.ForeignKey(FileRecord, on_delete=models.CASCADE)
    unit_de = models.CharField(max_length=4000, blank=True, null=True)
    unit_en = models.CharField(max_length=4000, blank=True, null=True)
    epsg_code = models.CharField(max_length=4000, blank=True, null=True)
    no_data_value = models.IntegerField(default=-9999)
    axis_label_x = models.CharField(max_length=4000, blank=True, null=True)  # is readonly
    axis_label_y = models.CharField(max_length=4000, blank=True, null=True)  # is readonly
    variable = models.ForeignKey(DataVariable, on_delete=models.CASCADE, null=True)
    scale_factor = models.IntegerField(default=1)
    wms_url = models.URLField(max_length=4000, null=True)  # is readonly
    layer_name = models.CharField(max_length=4000)  # is readonly
    time_steps = ArrayField(models.CharField(max_length=100, blank=True, null=True), blank=True, null=True, default=list)  # is readonly
    uncertainty_path = models.CharField(max_length=4000, blank=True, null=True)
    opacity = models.DecimalField(max_digits=3, decimal_places=2, default=0.5)
    is_point_request_enabled = models.BooleanField(default=True)

    def __str__(self):
        if self.name_en:
            return self.name_de
        else:
            return 'TBD (' + self.file_record.bucket.name + ' ' + str(self.id) + ')'


class GeojsonLayer(Layer):
    file_record = models.ForeignKey(FileRecord, on_delete=models.CASCADE)
    geonetwork_uuid = models.UUIDField(blank=True, null=True)


class AreaLayer(models.Model):
    area = models.ForeignKey(Area, on_delete=models.CASCADE, blank=True, null=True)
    position = models.IntegerField(default=1)
    is_active = models.BooleanField(default=False)
    is_default_on_start = models.BooleanField(default=False)
    wms_layer = models.ForeignKey(WmsLayer, on_delete=models.CASCADE, blank=True, null=True)
    sta_layer = models.ForeignKey(StaLayer, on_delete=models.CASCADE, blank=True, null=True)
    geojson_layer = models.ForeignKey(GeojsonLayer, on_delete=models.CASCADE, blank=True, null=True)

    class Meta:
        constraints = [
            models.UniqueConstraint(fields=['area', 'position'], name='unique_area_layer'),
        ]

    def __str__(self):
        if self.wms_layer:
            return str(self.wms_layer)
        elif self.sta_layer:
            return str(self.sta_layer)
        elif self.geojson_layer:
            return str(self.geojson_layer)
        else:
            return 'TBD (' + str(self.id) + ')'


class WmsLayerLegend(models.Model):
    wms_layer = models.ForeignKey(WmsLayer, on_delete=models.CASCADE)
    legend = models.ForeignKey(WmsLegend, on_delete=models.CASCADE)
    replace_values_in_viewer_by_label = models.BooleanField(default=False)
    position = models.IntegerField(default=1)

    class Meta:
        constraints = [
            models.UniqueConstraint(fields=['wms_layer', 'legend'], name='unique_wms_layer_legend'),
            models.UniqueConstraint(fields=['wms_layer', 'position'], name='unique_wms_legend_pos')
        ]


class Process(CommonInfo):
    type = models.CharField(max_length=20, choices=COMPUTATION_TYPES)
    sub_type = models.CharField(max_length=100, unique=True)
    spatial_aggregation_type = models.CharField(max_length=32, blank=True, null=True)
    temporal_aggregation_type = models.CharField(max_length=32, blank=True, null=True)

    def __str__(self):
        if self.name_de:
            return self.name_de
        else:
            return self.type + ':' + self.sub_type

    class Meta:
        verbose_name = 'Process'
        verbose_name_plural = 'Processes'


class WmsLayerProcess(models.Model):
    wms_layer = models.ForeignKey(WmsLayer, on_delete=models.CASCADE)
    process = models.ForeignKey(Process, on_delete=models.CASCADE)
    in_cache = models.BooleanField(default=False)

    class Meta:
        constraints = [
            models.UniqueConstraint(fields=['wms_layer', 'process'], name='unique_layer_comp')
        ]

    class Meta:
        verbose_name = 'WmsLayer-Process-Configuration'
        verbose_name_plural = 'WmsLayer-Process-Configurations'


class ProcessCache(models.Model):
    wms_layer = models.ForeignKey(WmsLayer, on_delete=models.CASCADE)
    process = models.ForeignKey(Process, on_delete=models.CASCADE)
    polygon_id = models.CharField(max_length=100)
    time_steps = ArrayField(models.CharField(max_length=100))
    results = models.JSONField()

    class Meta:
        constraints = [
            models.UniqueConstraint(fields=['wms_layer', 'process', 'polygon_id'], name='unique_layer_process_polygon')
        ]


class ConnectedLayer(models.Model):
    wms_layer = models.ForeignKey(WmsLayer, on_delete=models.CASCADE, related_name='ref_layer')
    referenced_wms_layer = models.ForeignKey(WmsLayer, on_delete=models.CASCADE, related_name='ref_target_layer')
    type = models.CharField(max_length=20, choices=REQUEST_TYPES, default='')

    class Meta:
        constraints = [
            models.UniqueConstraint(fields=['wms_layer', 'referenced_wms_layer'], name='unique_connected_wms_layer')
        ]


class BucketEvent(models.Model):
    event_time = models.DateTimeField()
    event_data = models.JSONField()
    job_status = models.IntegerField(blank=True, null=True)
    logs = models.CharField(max_length=4000, blank=True, null=True)
    is_new = models.BooleanField(default=False, null=True)
    has_metadata_record = models.BooleanField(default=False, null=True)


class ListFilter(CommonInfo):
    area = models.ForeignKey(Area, on_delete=models.CASCADE)
    position = models.IntegerField(default=1)
    is_active = models.BooleanField(default=True)

    class Meta:
        verbose_name = 'List-Filter'
        verbose_name_plural = 'List-Filter'
        constraints = [
            models.UniqueConstraint(fields=['area', 'position'], name='unique_filter_pos')
        ]


class ListFilterItem(CommonInfo):
    list_filter = models.ForeignKey(ListFilter, on_delete=models.CASCADE)

    class Meta:
        verbose_name = 'Filter-Option'
        verbose_name_plural = 'Filter-Options'

    def __str__(self):
        return self.list_filter.name_de + ' - ' + self.name_de


class LayerListFilterItem(models.Model):
    layer = models.ForeignKey(AreaLayer, on_delete=models.CASCADE)
    filter_item = models.ForeignKey(ListFilterItem, on_delete=models.CASCADE)

    class Meta:
        verbose_name = 'Connected Layer'
        verbose_name_plural = 'Connected Layers'
        constraints = [
            models.UniqueConstraint(fields=['layer', 'filter_item'], name='unique_layer_list_filter')
        ]

    def __str__(self):
        return 'Layer ' + str(self.layer.id)
