import os
import requests
import json

from zipfile import ZipFile
from io import BytesIO

from urllib.request import urlopen
from django.views.decorators.csrf import csrf_exempt

from django.shortcuts import render
from django.http import HttpResponse, HttpResponseRedirect, JsonResponse
from django.shortcuts import get_object_or_404
from django.forms.models import model_to_dict
from requests.auth import HTTPBasicAuth

from .models import *
from main.lib.api.AggregationApi import AggregationApi
from main.lib.utils.s3_utils import get_minio_download_link, get_s3_filesystem, _get_minio_client, list_minio_folder
from main.lib.utils.backend import create_legend_map


def set_text_by_locale(record, locale, key=None):
    if key:
        record[key] = record[key + '_en'] if locale == "en" else record[key + '_de']

        for prop in [key + '_de', key + '_en']:
            del record[prop]
    else:
        record['name'] = record['name_en'] if locale == "en" else record['name_de']

        for prop in ['name_de', 'name_en']:
            del record[prop]

    return record


def get_projects(request, locale):
    projects = Project.objects.filter(show_in_gallery=True).order_by('position_in_gallery')

    result = {'message': '', 'success': True, 'result': {}}
    project_overview = []

    for project in projects:
        project_overview.append(_get_project_information(project, locale))

    result['result']['projects'] = project_overview
    return JsonResponse(result)


def get_project_information(request, locale, project_name):
    project = get_object_or_404(Project, name_in_backend=project_name)

    result = {'message': '', 'success': True, 'result': {}}

    record = _get_project_information(project, locale)

    result['result']['project_info'] = record

    return JsonResponse(result)


def get_project_areas(request, locale, project_name):
    project = get_object_or_404(Project, name_in_backend=project_name)

    result = {'message': '', 'success': True, 'result': {}}
    result['result']['areas'] = []

    for area in Area.objects.filter(project=project.id, is_active=True).order_by('position'):

        area_record = model_to_dict(area)
        area_record['layers'] = []

        area_record = set_text_by_locale(area_record, locale)
        area_record = set_text_by_locale(area_record, locale, 'info')

        for area_layer in AreaLayer.objects.filter(area=area.id, is_active=True).order_by('position'):

            layer_record = {}

            if area_layer.wms_layer:
                layer_record = get_wms_layer(area_layer, locale)

            if area_layer.sta_layer:
                layer_record = get_sta_layer(area_layer, locale)

            if area_layer.geojson_layer:
                layer_record = get_geojson_layer(area_layer, locale)

            layer_record['is_default_on_start'] = area_layer.is_default_on_start
            area_record['layers'].append(layer_record)

        result['result']['areas'].append(area_record)

    return JsonResponse(result)


def get_wms_layer(area_layer: AreaLayer, locale: str):
    wms_layer = area_layer.wms_layer

    layer_record = model_to_dict(wms_layer)
    for prop in ['variable', 'file_record']:
        del layer_record[prop]

    layer_record = set_text_by_locale(layer_record, locale)
    layer_record = set_text_by_locale(layer_record, locale, 'info')
    layer_record = set_text_by_locale(layer_record, locale, 'unit')

    for prop in ['styles', 'available_aggregations', 'available_methods', 'connected_timeseries_layer',
                 'connected_single_value_layer']:
        layer_record[prop] = []

    layer_record['legends_to_map'] = {}

    for layer_legend in WmsLayerLegend.objects.filter(wms_layer=wms_layer.id).order_by('position'):

        legend_record = model_to_dict(layer_legend.legend)
        legend_record = set_text_by_locale(legend_record, locale)

        legend_name = legend_record['name']

        layer_record['styles'].append(legend_name)
        if layer_legend.replace_values_in_viewer_by_label:
            layer_record['legends_to_map'][legend_name] = create_legend_map(layer_legend.legend, locale)

    for layer_process in WmsLayerProcess.objects.filter(wms_layer=wms_layer.id):
        if layer_process.process.type == AGGREGATION:
            layer_record['available_aggregations'].append(layer_process.process.sub_type)
        if layer_process.process.type == ANOMALY or layer_process.process.type == MAGNITUDE:
            layer_record['available_methods'].append(layer_process.process.type)

    if len(layer_record['available_aggregations']) > 0:
        layer_record['available_methods'].append(AGGREGATION)

    for connected_layer in ConnectedLayer.objects.filter(wms_layer=wms_layer.id):
        ref_area_layer = AreaLayer.objects.get(wms_layer_id=connected_layer.referenced_wms_layer)
        if connected_layer.type == REQUEST_TYPE_1:
            layer_record['connected_timeseries_layer'].append(ref_area_layer.id)
        if connected_layer.type == REQUEST_TYPE_2:
            layer_record['connected_single_value_layer'].append(ref_area_layer.id)

    layer_record['id'] = area_layer.id
    layer_record['type'] = 'wms'

    return layer_record


def get_sta_layer(area_layer: AreaLayer, locale: str):
    sta_layer = area_layer.sta_layer

    layer_record = model_to_dict(sta_layer)

    layer_record = set_text_by_locale(layer_record, locale)
    layer_record = set_text_by_locale(layer_record, locale, 'info')

    layer_record['id'] = area_layer.id
    layer_record['type'] = 'sta'

    layer_record['legend'] = []
    layer_legend = sta_layer.legend
    if layer_legend:
        for color in StaColor.objects.filter(legend=layer_legend).order_by('-position'):
            color_record = model_to_dict(color)
            del color_record['id']
            del color_record['legend']
            layer_record['legend'].append(color_record)

    return layer_record


def get_geojson_layer(area_layer: AreaLayer, locale: str):
    geojson_layer = area_layer.geojson_layer

    layer_record = model_to_dict(geojson_layer)

    layer_record = set_text_by_locale(layer_record, locale)
    layer_record = set_text_by_locale(layer_record, locale, 'info')

    base_url = os.environ.get('MINIO_ENDPOINT')
    if base_url.endswith('/'):
        base_url = base_url.rstrip('/')

    file_path = geojson_layer.file_record.file_path
    if file_path.startswith('/'):
        file_path = file_path.lstrip('/')

    layer_record['url'] = '{}/{}'.format(base_url, file_path)

    layer_record['id'] = area_layer.id
    layer_record['type'] = 'geojson'

    return layer_record


def get_countries(request):
    with open("main/geojson/countries/2_hoch.geo.json") as f:
        data = json.load(f)
    return JsonResponse(data)


def get_districts_by_country(request, country_id):
    with open("main/geojson/districts/{}_admin.geojson".format(country_id)) as f:
        data = json.load(f)
    return JsonResponse(data)


def get_download(request, file_identifier):
    file_record = get_object_or_404(FileRecord, file_identifier=file_identifier)

    bucket_name = file_record.bucket.name
    file_path = file_record.file_path
    object_key = file_path.replace(bucket_name + '/', '')

    download_link = get_minio_download_link(bucket_name, object_key)
    return HttpResponseRedirect(download_link)


def get_zip_download(request, file_identifier):
    # database entry
    metadata_json = get_object_or_404(FileRecord, file_identifier=file_identifier)

    # base path (with bucket)
    base_path = _get_base_path(metadata_json.file_path)

    # data files as zip
    file_paths = _get_data_files(base_path, metadata_json)
    s3 = get_s3_filesystem()
    zipped_files = BytesIO()
    with ZipFile(zipped_files, 'w') as zip_file:
        for file in file_paths:
            s3_file = s3.open(file, 'rb')
            zip_file.writestr(file.removeprefix(base_path), s3_file.read())

    # metadata file name
    metadata_filename = _get_file_name(metadata_json.file_path)

    # response
    response = HttpResponse(zipped_files.getvalue())
    response['Content-Type'] = 'application/zip'
    response['Content-Disposition'] = 'attachment; filename="' + metadata_filename + '.zip"'
    return response


def _get_data_files(base_path, metadata_json):
    metadata_data_files = [metadata_json.file_path]
    for prefix in metadata_json.data_files:
        relative_path = base_path.removeprefix(metadata_json.bucket.name) + prefix
        data_files = list_minio_folder(metadata_json.bucket.name, relative_path.removeprefix('/'))
        for file in data_files:
            metadata_data_files.append(metadata_json.bucket.name + '/' + file.object_name)
    return metadata_data_files


def _get_base_path(file_path):
    splits = file_path.split('/')
    base_path = ''
    for i in range(0, len(splits) - 1):
        base_path = base_path + splits[i] + '/'
    return base_path


def _get_file_name(file_path):
    cut_type = file_path.split('.')[0]
    file = cut_type.split('/')
    return file[len(file) - 1]


def get_uncertainty(request, file_path):
    s3 = get_s3_filesystem()
    s3_file = s3.open(file_path, mode='rb')
    return HttpResponse(s3_file)


def _get_layer(layer_id):
    try:
        layer = AreaLayer.objects.get(id=layer_id)
        return layer.wms_layer
    except AreaLayer.DoesNotExist:
        return None


def _get_process(type, sub_type):
    try:
        return Process.objects.get(type=type, sub_type=sub_type)
    except Process.DoesNotExist:
        return None


@csrf_exempt
def call_process(request):
    body = json.loads(request.body)
    proc_api = AggregationApi(os.environ.get("AGGREGATION_API_URL"))

    layer = _get_layer(body['layer_id'])
    process = _get_process(body['process_type'], body['process_sub_type'])

    if layer is not None and process is not None:
        cache_result = proc_api.get_cache(layer, process, body['polygon'], body['time_steps'])
        if cache_result is not None:
            cache_result['status'] = True
            return JsonResponse(cache_result)

        results = proc_api.call_process(layer, process, body['polygon'], body['time_steps'])
        if results is not None:
            results['status'] = True
            return JsonResponse(results)

    return JsonResponse({'status': False})


def proxy_geoserver_request(request):
    url = os.environ.get("GEOSERVER_URL") + '/ows'

    if request.method == 'GET':
        response = urlopen(url + '?' + request.GET.urlencode())
        headers = response.headers
        return HttpResponse(response, content_type=headers.get_content_type())


def get_filters(request, locale: str, area_id: int):

    result = {'message': '', 'success': True, 'list_filters': []}

    for list_filter in ListFilter.objects.filter(area=area_id, is_active=True).order_by('position'):
        filter = model_to_dict(list_filter)
        filter['items'] = []

        filter = set_text_by_locale(filter, locale)

        for list_filter_item in ListFilterItem.objects.filter(list_filter=list_filter):
            item = model_to_dict(list_filter_item)
            item['layers'] = []

            item = set_text_by_locale(item, locale)

            for assigned_layer in LayerListFilterItem.objects.filter(filter_item=list_filter_item):
                item['layers'].append(assigned_layer.layer.id)

            filter['items'].append(item)

        result['list_filters'].append(filter)

    return JsonResponse(result)


def _get_project_information(project, locale):
    record = model_to_dict(project)
    record = set_text_by_locale(record, locale)
    record = set_text_by_locale(record, locale, 'teaser')
    record = set_text_by_locale(record, locale, 'about_text')
    record = set_text_by_locale(record, locale, 'imprint_text')
    record = set_text_by_locale(record, locale, 'faq_text')
    record = set_text_by_locale(record, locale, 'maintenance_notification')
    return record