import {defineComponent, onMounted, PropType, ref} from "vue"
import { Coordinate } from 'openlayers'
import axios from 'axios'
import proj4 from 'proj4'
import MapLegend from "@/components/MapLegend.vue"
import WmsLayer from "@/components/wms/WmsLayer.vue"
import WmsTimeControl from "@/components/wms/WmsTimeControl.vue"
import ShapefilePolygon from "@/components/wms/ShapefilePolygon.vue"
import FilterSelect from "@/components/FilterSelect.vue"
import MapPrint from "@/components/MapPrint.vue";
import {
    Area,
    MapClickEvent,
    Vue3SwipeControl,
    Vue3TileLayer,
    Layer,
    STALayer,
    WMSLayer,
    LayerFilter,
    TimeseriesEvent,
    AggregationEvent,
    CovJsonSingleResult,
    PlotlyTimeSeries,
    STAObservedProperty,
    STAFilterParams,
    StaThing,
    GEOJSONLayer
} from "@/types"
import { parseMarkdown } from "@/composables/parseMarkdown"
import { getLastTimeStepIfExists, setPageHeight } from "@/composables/utils"
import { countryCenterCoords } from "@/composables/countries"
import shp, { FeatureCollectionWithFilename, ShpJSBuffer } from 'shpjs'
import OpacityControl from "@/components/wms/OpacityControl.vue";
import StaLayer from "@/components/sta/StaLayer.vue";
import AggregationWidget from "@/components/wms/AggregationWidget.vue";
import LayerMenu from "@/components/LayerMenu.vue";
import TimeSeriesChart from "@/components/TimeSeriesChart.vue";
import MapPopup from "@/components/MapPopup.vue";
import GeojsonLayer from "@/components/geojson/GeojsonLayer.vue";
import {convertTime} from "@/composables/time_utils";
import ProjectTeaser from "@/components/ProjectTeaser.vue";
import StaFilter from "@/components/sta/StaFilter.vue";
import ShadowMapPopup from "@/components/ShadowMapPopup.vue";

proj4.defs("EPSG:32632", "+proj=utm +zone=32 +datum=WGS84 +units=m +no_defs")
proj4.defs("EPSG:3035","+proj=laea +lat_0=52 +lon_0=10 +x_0=4321000 +y_0=3210000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs +type=crs");

export default defineComponent({
    components: {
        ShadowMapPopup,
        ProjectTeaser,
        MapPopup,
        LayerMenu,
        AggregationWidget,
        StaLayer,
        FilterSelect,
        StaFilter,
        MapLegend,
        OpacityControl,
        GeojsonLayer,
        ShapefilePolygon,
        WmsTimeControl,
        TimeSeriesChart,
        WmsLayer,
        MapPrint
    },
    props: {
        teaser: String,
        zoom: Number,
        center: Object as PropType<Coordinate>,
        projection: String,
        areas: {
            type: Object as PropType<Area[]> | undefined,
            required: true
        },
        backendUrl: String,
        menuSidenavToggle: Boolean
    },
    data() {
        return {
            activeArea: this.$props.areas[0] as Area,
            timeseriesEvent: null as TimeseriesEvent|null,
            aggregationEvent: null as AggregationEvent|null,

            isPopupEnabled: false as boolean,
            isChartHidden: false as boolean,
            isChartEnabled: false as boolean,
            isAggregationWidgetEnabled: false as boolean,
            showAreaInfoBox: false as boolean,
            isWms1Loading: false as boolean,
            isWms2Loading: false as boolean,
            isStaLoading: false as boolean,
            isGeojsonLoading: false as boolean,
            isChartLoading: false as boolean,

            areCountriesVisible: false as boolean,
            areDistrictsVisible: false  as boolean,
            isShapefileVisible: false as boolean,
            hoveredFeature: '' as string,
            hoveredFeatureId: '' as string,
            selectedFeature: '' as string,
            selectedFeatureId: '' as string,

            shapefileGeoJson: {} as FeatureCollectionWithFilename,
            projWarning: '' as string,

            stabilityUncertainty: undefined as string | undefined,
            significanceUncertainty: undefined as string | undefined,
            showStabilityUncertainty: false as boolean,
            showSignificanceUncertainty: false as boolean,
            showStaLayer: true as boolean,

            showSideNav: false as boolean,
            showAreaMenu: false as boolean,
            showMainFilterSelect: false as boolean,
            showComparisonFilterSelect: false as boolean,
            showLayerInfo: false as boolean,
            showMainLayerTimeRange: true as boolean,
            layerInfoTitle: '' as string,
            layerInfoText: '' as string,
            listFilters: [],
            mainFilters: {} as LayerFilter,
            comparisonFilters: {} as LayerFilter,
            selectableMainLayers: [] as Layer[],
            selectableComparisonLayers: [] as WMSLayer[],

            mainLayerOpacity: 0.5 as number,
            comparisonLayerOpacity: 0.5 as number,

            wmsSingleResult: {} as CovJsonSingleResult,
            staTimeSeries: {} as PlotlyTimeSeries,

            isNewStaEvent: false as boolean,
            staObsProperty: {} as STAObservedProperty,
            staInput: {} as STAFilterParams,
            showStaFilter: false as boolean,

            showPrintDialog: false as boolean,
            showPointRequestMessage: false as boolean,
            messageCoordinate: [0,0] as Coordinate,

            stationInformation: undefined as StaThing | undefined,

            showShadowMapPopup: false as boolean,
            previousTimeseriesEvent: null as TimeseriesEvent|null,
            previousWmsSingleResult: {} as CovJsonSingleResult,

            hasAreaStaLayers: false as boolean,
            hasAreaGeojsonLayers: false as boolean
        }
    },
    computed: {
        getInfoBoxText() {
            return parseMarkdown(this.activeArea.info)
        },
        getDistrictWfsRoute() {
            // zum Zeitpunkt der Abfrage ist eine fünfstellige Landkreisnummer die ID mit den ersten zwei Ziffern als Bundesland-Nummer
            return this.backendUrl + 'districts/' + this.selectedFeatureId.slice(0,2)
        },
        showFilterToggle() {
            return (this.listFilters.length > 0)
        },
        isActiveMainFilter() {
            return (Object.keys(this.mainFilters).length > 0)
        },
        isActiveComparisonFilter() {
            return (Object.keys(this.comparisonFilters).length > 0)
        },
        isWideScreen() {
            return (window.innerWidth > 1408)
        },
        isTablet() {
            return (window.innerWidth > 769)
        },
        isAggregationMode() {
            if (this.areCountriesVisible || this.areDistrictsVisible || this.isShapefileVisible) {
                return true
            }
            return false
        },
        widgetMode() {
            return {
                'modal': !this.isWideScreen,
                'is-active': (this.isChartEnabled && !this.isChartHidden) || this.isAggregationWidgetEnabled
            }
        },
        attribution() {
            return '© Bundesamt für Kartographie und Geodäsie ' + new Date().getFullYear()
        }
    },
    methods: {
        convertTime,
        openTimeSeriesChart() {
            this.isChartEnabled = true
            this.isChartHidden = false
        },
        turnOffChartAndPopup() {
            if (!this.isWideScreen) {
                this.isChartHidden = true
            } else {
                this.isChartHidden = false
                this.isPopupEnabled = false
                this.isChartEnabled = false
            }
        },
        turnOffAggregationWidget() {
            this.isAggregationWidgetEnabled = false
            this.aggregationEvent = null
        },
        toggleCountries() {
            this.showSwiper = false
            this.areCountriesVisible = !this.areCountriesVisible
            this.areDistrictsVisible = false
            this.isShapefileVisible = false
            this.isPopupEnabled = false
            this.isChartEnabled = false
            this.aggregationEvent = null
        },
        showDistricts() {
            this.showSwiper = false
            this.areCountriesVisible = false
            this.areDistrictsVisible = true
            this.isShapefileVisible = false
            this.isPopupEnabled = false
            this.isChartEnabled = false
        },
        switchArea(areaIndex: number, layerIndex: number = 0) {
            this.showPrintDialog = false
            this.showSwiper = false
			this.activeArea = this.$props.areas[areaIndex]

            if (layerIndex === 0) {
                layerIndex = this.getIndexOfDefaultLayer(areaIndex)
            }

            this.setDefaultLayerFromIndex(layerIndex)

            if (this.activeArea.layers.length > 1) {
                this.setComparisonLayerFromIndex(0) // TODO
            }

            this.clearMainFilter()
            this.clearComparisonFilter()
            this.showMainFilterSelect = false
            this.showComparisonFilterSelect = false
            this.requestFilters()
            this.selectableMainLayers = this.activeArea.layers
            this.selectableComparisonLayers = this.activeArea.layers

            this.aggregationEvent = null

            this.showAreaMenu = false
            //this.timeseriesEvent = {}
            this.staTimeSeries = {}

            this.unsetStaLayer()
            this.unsetGeojsonLayer()

            for (const layer of this.activeArea.layers) {
                if (layer.type === 'sta') {
                    this.hasAreaStaLayers = true
                }
                if (layer.type === 'geojson') {
                    this.hasAreaGeojsonLayers = true
                }
            }
        },
        toggleSwiper() {
            this.showPrintDialog = false
            this.isPopupEnabled = false
            this.isChartEnabled = false
            this.areCountriesVisible = false
            this.areDistrictsVisible = false
            this.isShapefileVisible = false
            this.turnOffAggregationWidget()
            this.showSwiper = !this.showSwiper

            this.showStaLayer = !this.showSwiper

            this.showStabilityUncertainty = false
            this.showSignificanceUncertainty = false
        },
        toggleOpacity() {
            this.mainLayerOpacity = (this.mainLayerOpacity !== 1) ? 1 : Number(this.mainLayer.opacity)
            if(this.showSwiper) {
                this.comparisonLayerOpacity = ( this.comparisonLayerOpacity !== 1) ? 1 : Number(this.comparisonLayer.opacity)
            }
            this.$refs.map.render() // otherwise opacity only changes after moving on map
        },
        toggleMainFilterSelect() {
            this.showPrintDialog = false
            this.showComparisonFilterSelect = false
            this.showMainFilterSelect = !this.showMainFilterSelect
        },
        toggleComparisonFilterSelect() {
            this.showPrintDialog = false
            this.showMainFilterSelect = false
            this.showComparisonFilterSelect = !this.showComparisonFilterSelect
        },
        toggleStaFilter() {
            this.showStaFilter = !this.showStaFilter
        },
        closePopup() {
            this.isPopupEnabled = false
            if (this.isWideScreen) {
                this.closeWidget()
            }
        },
        closeWidget() {
            if (this.isWideScreen) {
                this.isPopupEnabled = false
            }
        },
        togglePrintDialog() {
            this.showPrintDialog = !this.showPrintDialog
        },
        setDefaultLayer(layer: Layer) {
            if (layer.type == 'wms') {
                this.setMainLayer(layer as WMSLayer)
            } else if (layer.type == 'sta') {
                this.setStaLayer(layer as STALayer)
            } else if (layer.type == 'geojson') {
                this.setGeojsonLayer(layer as GEOJSONLayer)
            }
        },
        setMainLayer(mainLayer: WMSLayer) {
            this.showPrintDialog = false
            this.isPopupEnabled = false
            this.isChartEnabled = false
            this.mainLayer = mainLayer

            this.setMainLayerTime(getLastTimeStepIfExists(this.mainLayer))
            this.changeMainLayerStyle(this.mainLayer.styles[0])
            this.mainLayerOpacity = Number(mainLayer.opacity)

            this.aggregationEvent = null

            this.showStabilityUncertainty = false
            this.showSignificanceUncertainty = false
            this.setMainLayerUncertainties()

            this.setPreviousResults()
        },
        setStaLayer(layer: STALayer) {
            this.unsetStaLayer()
            setTimeout(() => this.staLayer = layer, 100)
        },
        setGeojsonLayer(layer: GEOJSONLayer) {
            this.unsetGeojsonLayer()
            setTimeout(() => this.geojsonLayer = layer, 100)
        },
        unsetMainWmsLayer() {
            this.mainLayer = null
            this.isPopupEnabled = false
            this.isChartEnabled = false
        },
        unsetStaLayer() {
            this.staLayer = null
            this.stationInformation = undefined
            this.isPopupEnabled = false
            this.isChartEnabled = false
        },
        unsetGeojsonLayer() {
            this.geojsonLayer = null
        },
        setDefaultLayerFromIndex(layerIndex: number) {
            this.selectedRadioOption = null
            this.checkedFilter = {}
            this.setDefaultLayer(this.activeArea.layers[layerIndex])
        },
        getLayer(layerId: number){
            for (const layer of this.activeArea.layers) {
                if (layer.id === layerId) {
                    return layer
                }
            }
        },
        setComparisonLayer(comparisonLayer: WMSLayer) {
            this.isPopupEnabled = false
            this.isChartEnabled = false
            this.comparisonLayer = comparisonLayer
            if (comparisonLayer.type == 'wms') {
                this.setComparisonLayerTime(getLastTimeStepIfExists(this.comparisonLayer as WMSLayer))
                this.changeComparisonLayerStyle(this.comparisonLayer.styles[0])
                this.comparisonLayerOpacity = Number(comparisonLayer.opacity)
            }

            this.setPreviousResults()
        },
        setComparisonLayerFromIndex(index: number) {
            this.setComparisonLayer(this.activeArea.layers[index])
        },
        setMainLayerTime(value: string) {
            this.mainLayerTime = value
            this.isPopupEnabled = false
            this.isChartEnabled = false

            this.setPreviousResults()

            //this.resetPopupDisplay()
            this.setMainLayerUncertainties()
        },
        setStaObsProperty(property: STAObservedProperty) {
            this.closeWidget()
            this.closePopup()
            this.staObsProperty = property
        },
        setStaInput(params: STAFilterParams) {
            this.staInput = params
        },
        removeShapefile() {
            this.isShapefileVisible = false
            this.$refs.fileInput.value = null
            this.shapefileGeoJson = {}
        },
        setMainLayerUncertainties() {
            this.stabilityUncertainty = undefined
            this.significanceUncertainty = undefined
            if (this.mainLayer.uncertainty_path) {
                this.stabilityUncertainty = this.mainLayer.uncertainty_path + "_direction0p66_timestamp" + this.mainLayer.time_steps.indexOf(this.mainLayerTime) + ".geoJSON"
                this.significanceUncertainty = this.mainLayer.uncertainty_path + "_significance0p5_timestamp" + this.mainLayer.time_steps.indexOf(this.mainLayerTime) + ".geoJSON"
            }
        },
        toggleUncertaintyDir() {
            this.showStabilityUncertainty = !this.showStabilityUncertainty
        },
        toggleUncertaintySig() {
            this.showSignificanceUncertainty = !this.showSignificanceUncertainty
        },
        toggleTimeRange() {
            this.showMainLayerTimeRange = !this.showMainLayerTimeRange
        },
        setComparisonLayerTime(value: string) {
            this.comparisonLayerTime = value
            this.isPopupEnabled = false
            this.isChartEnabled = false

            this.setPreviousResults()
        },
        toggleSideNav() {
            this.showSideNav = !this.showSideNav
        },
        toggleAreaMenu() {
            this.showAreaMenu = !this.showAreaMenu
        },
        toggleLayerInfo(layer: Layer|null = null) {
            this.showLayerInfo = !this.showLayerInfo
            if (layer) {
                this.layerInfoTitle = layer.name
                this.layerInfoText = parseMarkdown(layer.info)
            }
        },
        toggleAreaInfoBox() {
            this.showAreaInfoBox = !this.showAreaInfoBox
        },
        setCenterOfCountry() {
            if (this.selectedFeatureId == '') {
                return;
            }
            const coord = countryCenterCoords[this.selectedFeatureId] as Coordinate
            const bl_center = proj4('WGS84', 'EPSG:3857', coord)

            this.$refs.view.animate({zoom: 7.5, center: bl_center})
        },
        requestFilters() {

            const locale = (this.$i18n.locale) ? this.$i18n.locale : this.$i18n.fallbackLocale

            const url = this.backendUrl + 'filter/' + locale + '/' + this.activeArea.id

            this.listFilters = []

            axios.get(url)
                .then(response => {
                    this.createFilters(response.data)
                })
                .catch((error) => {
                    console.log(error)
                })
        },
        createFilters(responseData: any) {
            for (const filter of responseData['list_filters']) {
                this.listFilters.push(filter)
            }
        },
        filterMainLayers(filterId: string, layers: number[]) {

            this.mainFilters[filterId] = layers

            if (layers.length > 0) {
                this.selectableMainLayers = this.collectSelectableLayers(this.mainFilters, [])
            }
        },
        removeMainFilter(filterId: string) {
            delete this.mainFilters[filterId]
            this.selectableMainLayers = this.collectSelectableLayers(this.mainFilters, [])
        },
        clearMainFilter() {
            this.mainFilters = {}
            this.selectableMainLayers = this.activeArea.layers
        },
        filterComparisonLayers(filterId: string, layers: number[]) {

            this.comparisonFilters[filterId] = layers

            if (layers.length > 0) {
                this.selectableComparisonLayers = this.collectSelectableLayers(this.comparisonFilters, [])
            }
        },
        removeComparisonFilter(filterId: string) {
            delete this.comparisonFilters[filterId]
            this.selectableComparisonLayers = this.collectSelectableLayers(this.comparisonFilters, [])
        },
        clearComparisonFilter() {
            this.comparisonFilters = {}
            this.selectableComparisonLayers = this.activeArea.layers
        },
        collectSelectableLayers(filters: LayerFilter, layers: Layer[]) {
            const iteratedLayers: number[] = []

            if (Object.keys(filters).length == 0) {
                layers = this.activeArea.layers
            } else {
                for (const listFilterName of Object.keys(filters)) {

                    for (const layerId of filters[listFilterName]) {

                        // collect layer only if it is in all currently active filters
                        if (!this.isLayerInAllFilters(filters, layerId)) {
                            continue
                        }

                        // collect layer only once, so it will be only once in the sidenav
                        if(iteratedLayers.indexOf(layerId) != -1) {
                            continue
                        }

                        iteratedLayers.push(layerId)
                        const layer = this.getLayer(layerId)
                        layers.push(layer)
                    }
                }
            }
            return layers
        },
        isLayerInAllFilters(filters: LayerFilter, layerId: number) {

            let result = true

            for (const listFilterName of Object.keys(filters)) {
                const layers = filters[listFilterName]

                if (layers.length > 0 && layers.indexOf(layerId) == -1) {
                    result = false
                }
            }
            return result
        },
        setPreviousResults() {
            this.previousTimeseriesEvent = this.timeseriesEvent
            this.previousWmsSingleResult = this.wmsSingleResult
        },
        initMapClickProcess(event: MapClickEvent) {
            this.showPrintDialog = false

            this.turnOffChartAndPopup() // reset timeseries data
            this.wmsSingleResult = {}

            this.processMapClick(event)
        },
        processMapClick(event: MapClickEvent) {

            this.setPreviousResults()

            if (this.isNewStaEvent) {
                this.isNewStaEvent = false
            } else {
                // reset because it is no click on sta-feature (and do not show old sta-timeseries again)
                this.staTimeSeries = {}
            }

            let layer: WMSLayer = this.mainLayer
            let layerTime: string = this.mainLayerTime
            if (this.swipeControl !== null) {
                if (this.isClickOnComparisonLayer(event)) {
                    layer = this.comparisonLayer
                    layerTime = this.comparisonLayerTime
                }
            }

            if (this.isAggregationMode) {
                this.isAggregationWidgetEnabled = true

                this.selectedFeature = this.hoveredFeature
                this.selectedFeatureId = this.hoveredFeatureId

                if(this.areCountriesVisible) {
                    this.setCenterOfCountry()
                    this.triggerAggregationWidget(layer, this.selectedFeature)
                } else if (this.areDistrictsVisible) {
                    this.triggerAggregationWidget(layer, this.selectedFeature)
                } else if (this.isShapefileVisible) {
                    this.triggerAggregationWidget(layer, 'shapeFile')
                }
            } else {

                if (layer) {
                    if (!layer.is_point_request_enabled) {
                        this.showPointRequestMessage = true
                        this.messageCoordinate = event.coordinate
                        setTimeout(() => this.showPointRequestMessage = false, 4000)
                        return
                    }
                }

                this.isPopupEnabled = true

                if(!this.staLayer) {
                    // Chart should not be displayed immediately because it distracts from reading the StationInformation in MapPopup
                    this.isChartEnabled = true
                }

                const connected_timeseries_layers: WMSLayer[] = []

                layer?.connected_timeseries_layer.forEach(function (layerId: number) {
                    const connected_layer: WMSLayer|null = this.findLayerByID(layerId)
                    if (connected_layer) {
                        connected_timeseries_layers.push(connected_layer)
                    }
                }.bind(this))

                const connected_single_value_layers: WMSLayer[] = []

                layer?.connected_single_value_layer.forEach(function (layerId: number) {
                    const connected_layer: WMSLayer|null = this.findLayerByID(layerId)
                    if (connected_layer) {
                        connected_single_value_layers.push(connected_layer)
                    }
                }.bind(this))

                this.timeseriesEvent = {
                    layer: layer,
                    layerTime: layerTime,
                    connected_timeseries_layers: connected_timeseries_layers,
                    connected_single_value_layers: connected_single_value_layers,
                    coordinate: event.coordinate,
                    positionWgs84: proj4('EPSG:3857', 'WGS84', event.coordinate)
                }
            }
        },
        triggerAggregationWidget(layer: WMSLayer, type: string) {
            this.aggregationEvent = {
                layer: layer,
                feature: type
            }
        },
        isClickOnComparisonLayer(e: MapClickEvent) {
            const event = e as MapClickEvent    // workaround: native object MapBrowserEvent has incorrect typing (frameState is missing)

            const mapSize: number = event.frameState.size[0]
            const clickedPointX: number = event.pixel[0]    // 'pixel' contains X and Y of coordinate related to map size

            const clickedPos = clickedPointX / mapSize
            const swiperPos = this.swipeControl.control.values_.position   // returns position of swiper in percent

            if (clickedPos > swiperPos) {
                // clicked point is on right side of swiper => and is therefore on the comparison layer
                return true
            }
            return false
        },
        findLayerIndexByID(id: number, areaIndex: number) {
            let layerIndex = 0
            this.$props.areas.forEach(function (area: Area) {
                area.layers.forEach((layer: Layer, index) => {
                    if (layer.id == id) {
                        layerIndex = index
                    }
                })
            })

            if (layerIndex == 0) {
                layerIndex = this.getIndexOfDefaultLayer(areaIndex)
            }
            return layerIndex
        },
        getIndexOfDefaultLayer(areaIndex: number) {
            let layerIndex = 0

            this.$props.areas[areaIndex].layers.forEach(function (layer: Layer, index: number) {
                if (layer.is_default_on_start) {
                    layerIndex = index
                }
            })
            return layerIndex
        },
        findLayerByID(id: number) {
            let result = null
            this.$props.areas.forEach(function (area: Area) {
                area.layers.forEach((layer: Layer) => {
                    if (layer.id == id) {
                        result = layer
                    }
                })
            })
            return result
        },
        findAreaIndexByID(id: number) {
            const areas = this.$props.areas
            const index = areas.findIndex((area: Area) => area.id == id)

            if (index == -1) {
                let defaultIndex = 0
                areas.forEach(function (area: Area, index: number) {
                    if (area.is_default_on_start) {
                        defaultIndex = index
                    }
                })
                return defaultIndex
            }
            return index
        },
        updateURL() {
            if (this.mainLayer !== null) {
                const params = new URLSearchParams();
                params.set('area', this.activeArea.id);
                params.set('layer', this.mainLayer.id);
                this.$router.replace("?" + params.toString())
            } else {
                this.$router.replace("")
            }
        },
        setWms1Loading(isLoading: boolean) {
            this.isWms1Loading = isLoading
        },
        setWms2Loading(isLoading: boolean) {
            this.isWms2Loading = isLoading
        },
        setVectorDataLoading(isLoading: boolean) {
            this.isGeojsonLoading = isLoading
        },
        setChartLoading(isLoading: boolean) {
            this.isChartLoading = isLoading
        },
        setFeature(name: string) {
            this.hoveredFeature = name
        },
        setFeatureId(id: string) {
            this.hoveredFeatureId = id
        },
        setShapefileGeoJson(geoJson: FeatureCollectionWithFilename | FeatureCollectionWithFilename[]) {
            this.areCountriesVisible = false
            this.areDistrictsVisible = false
            this.showSwiper = false
            this.isPopupEnabled = false
            this.isShapefileVisible = true
            this.shapefileGeoJson = geoJson

            this.showStabilityUncertainty = false
            this.showSignificanceUncertainty = false
        },
        async uploadShapefile(event: any) {
            if(event == undefined)
                return

            const files = event.target.files

            this.isShapefileVisible = true

            if (files[0]) {
                const file: any = files[0]
                const bufferResult: ShpJSBuffer = await file.arrayBuffer()

                shp.parseZip(bufferResult)
                .then((geojson: FeatureCollectionWithFilename | FeatureCollectionWithFilename[]) => {
                    this.setShapefileGeoJson(geojson)
                }).catch((error) => {
                    console.log(error)
                })
            }
        },
        setProjWarning(projWarning: string) {
            this.projWarning = projWarning
        },
        resetSwiper(layerComponent: any) {
            layerComponent.on('prerender', function (event: any) {
                const ctx = event.context;

                const mapSize = [window.innerWidth, window.innerHeight]
                const width = mapSize[0] * this.swipeControl.control.values_.position

                const tl = [width, 0]
                const tr = [mapSize[0], 0]
                const bl = [width, mapSize[1]]
                const br = mapSize

                ctx.restore();  // important, otherwise layer2 can only be clipped to the right
                ctx.save();
                ctx.beginPath();
                ctx.moveTo(tl[0], tl[1]);
                ctx.lineTo(bl[0], bl[1]);
                ctx.lineTo(br[0], br[1]);
                ctx.lineTo(tr[0], tr[1]);

                ctx.closePath();
                ctx.clip();

            }.bind(this))
        },
        setWmsSingleResult(data: CovJsonSingleResult) {
            this.wmsSingleResult = data
        },
        setStaTimeSeries(timeSeries: PlotlyTimeSeries) {
            this.isNewStaEvent = true
            this.staTimeSeries = timeSeries
        },
        setStationInformation(thing: any) {

          const url = process.env.VUE_APP_BACKEND_URL + 'sta-obs-properties-by-thing/' + this.staLayer.endpoint + '/' + thing["Thing/id"]

          const name_prop = (this.$i18n.locale == 'de') ? 'name_de' : 'name_en'

          axios.get(url)
              .then(response => {

                thing['selectableProperties'] = []

                for (const property of response.data.value) {
                  const obsProperty = property['ObservedProperty']

                    thing.selectableProperties.push({
                        id: obsProperty['@iot.id'],
                        dtxsid: obsProperty.properties.DTXSID,
                        casNr: obsProperty.properties['CAS_NR'],
                        inchiKey: obsProperty.properties['INCHI_KEY'],
                        fullName: obsProperty.properties[name_prop],
                        useGroup: obsProperty.properties['USE_GROUP']
                    })
                }
              })
              .catch((error: any) => {
                  console.log(error)
              })
              .finally(() => {
                  this.stationInformation = thing
              })
        }
    },
    mounted() {
        const areaID = this.$router.currentRoute.value.query.area
        const layerID = this.$router.currentRoute.value.query.layer
        const areaIndex = this.findAreaIndexByID(areaID)
        const layerIndex = this.findLayerIndexByID(layerID, areaIndex)

        this.mainLayer = null
        this.switchArea(areaIndex, layerIndex)

        setPageHeight()
        this.$refs.map.updateSize()

        window.addEventListener('resize', () => {
            setPageHeight()
            this.$refs.map.updateSize()
        })
    },
    created() {
        if (this.isWideScreen) {
            this.showSideNav = true
        }
    },
    watch: {
        activeArea() {
            this.updateURL()
        },
        mainLayer() {
            this.updateURL()
        },
        menuSidenavToggle() {
            this.showSideNav = !this.showSideNav
        }
    },
    setup(props) {
        let mainLayer = ref<WMSLayer | null>(null)
        let comparisonLayer = ref<WMSLayer | null>(null)

        const staLayer = ref<STALayer | null>(null)

        const geojsonLayer = ref<GEOJSONLayer | null>(null)

        const mainLayerTime = ref<string>('')
        const comparisonLayerTime = ref<string>('')

        const mainLayerStyle = ref<string>('')
        const comparisonLayerStyle = ref<string>('')

        for (const layer of props.areas[0].layers) {

            if (layer.type == 'wms') {
                const wmsLayer = layer as WMSLayer

                mainLayer = ref<WMSLayer>(wmsLayer)
                mainLayerTime.value = getLastTimeStepIfExists(wmsLayer)
                mainLayerStyle.value = wmsLayer.styles[0]

                comparisonLayer = ref<WMSLayer>(wmsLayer)
                comparisonLayerTime.value = getLastTimeStepIfExists(wmsLayer)

                comparisonLayerStyle.value = wmsLayer.styles[0] as string

                break
            }
        }

        const showSwiper = ref<boolean>(false)
        const swipeControl: Vue3SwipeControl = ref(null)

        const wmsLayer1: Vue3TileLayer = ref(null)
        const wmsLayer2: Vue3TileLayer = ref(null)

        const a: Array<Vue3TileLayer> = []
        const layerList = ref(a)


        const changeMainLayerStyle = (style: string) => {
            mainLayerStyle.value = style
        }
        const changeComparisonLayerStyle = (style: string) => {
            comparisonLayerStyle.value = style
        }

        onMounted(() => {
            layerList.value.push(wmsLayer1.value.olImageLayer.imageLayer)
            if (wmsLayer2.value !== null) {
                layerList.value.push(wmsLayer2.value.olImageLayer.imageLayer)
            }
        })

        const map = ref<string>()

        return {
            map,
            layerList,
            mainLayer,
            staLayer,
            geojsonLayer,
            comparisonLayer,
            wmsLayer1,
            wmsLayer2,
            showSwiper,
            swipeControl,
            mainLayerTime,
            comparisonLayerTime,
            mainLayerStyle,
            comparisonLayerStyle,
            changeMainLayerStyle,
            changeComparisonLayerStyle
        }
    }
})
