import { Coordinate } from 'openlayers'
import { SwipeControl, TileLayer } from "vue3-openlayers";


export type TimeRange = string[]
export type DateFormat = '' | 'period' | 'year' | 'month' | 'season'

export interface Area {
    id: number
    name: string
    activeBaseMap: string
    baseMaps: []
    layers: Layer[]
    info: string
    is_default_on_start: boolean
}

export interface ProjectInfo {
    id: number,
    name_in_url: string,
    name_in_backend: string,
    name: string,
    theme: string,
    teaser: string,
    about_text: string,
    imprint_text: string,
    faq_text: string,
    maintenance_notification: string,
    is_english_version_active: boolean,
    default_lon: number,
    default_lat: number,
    default_zoom: number,
    gallery_image: string
}


export interface Layer {
    id: number
    name: string
    info: string
    time_format: DateFormat
    y_axis_min: number
    y_axis_max: number
    type: 'wms' | 'sta' | 'geojson'
    is_default_on_start: boolean
}


export interface WMSLayer extends Layer {
    wms_url: string
    layer_name: string
    styles: string
    time_steps: string[]
    epsg_code: string
    axis_label_x: string
    axis_label_y: string
    unit: string
    desc: string
    no_data_value: number
    position: number
    available_aggregations: string[]
    available_methods: string[]
    scale_factor: number
    legends_to_map: {
        [key: string]: LegendRecord[]
    }
    connected_timeseries_layer: number[]
    connected_single_value_layer: number[]
    uncertainty_path: string | undefined
    opacity: number
    is_point_request_enabled: boolean
}


export interface STALayer extends Layer {
    endpoint: number
    observed_property_use_group: string
    legend: StaColor[]
}


export interface GEOJSONLayer extends Layer {
    url: string
}


export interface STAObservedProperty {
    id: number,
    dtxsid: string
    fullName: string
    casNr: string
    inchiKey: string
    useGroup: string
}


export interface STAFilterParams {
    useGroup: string,
    threshold: number,
    beginDate: string
    endDate: string
}


export interface StaThing {
    "Thing/description": string
    "Thing/id": number
    "Thing/name": string
    "Thing/properties/waterbody": string
    selectableProperties: STAObservedProperty[]
}


export interface MapClickEvent {
    coordinate: Coordinate
    pixel: [number, number]
    ranges: {
        [key: string]: {
            'values': number[]
        }
    }
    frameState: {
        size: [number, number]
    }
}

export interface CovJsonResponse {
    domain: {
        axes: {
            't': {
                'values': string[]
            }
        }
    }
    ranges: {
        [key: string]: {
            'values': number[]
        }
    }
}

export interface LegendRecord {
    color: string,
    label: string,
    quantity: number
}

export interface StringMap {
    [key: string]: string
  }

export interface Aggregation {
    [key: string]: number
}

export interface CountryCoordinateMap {
    [key: string]: Coordinate
}

export interface PlotlyTimeSeries {
  x: string[],
  y: number[],
  type: 'scatter',
  mode: 'lines+markers',
  name: string,
  position: number,
  timeFormat: DateFormat,
  unit: string|undefined,
  yMin: number|undefined,
  yMax: number|undefined,
  visible: boolean|string
}

export interface YaxisConfig {
    units: string[]
    yMin: number
    yMax: number
}

export interface CovJsonSingleResult {
  label: string,
  unit: string,
  value: number,
  time: string,
  textValue: string,
  error: boolean
}

export interface LayerFilter {
    [key: string]: number[]
}


export interface ColorLegend {
    label: string
    quantity: string
    color: string
}

export type Vue3SwipeControl = InstanceType<typeof SwipeControl>
export type Vue3TileLayer = InstanceType<typeof TileLayer>


export interface LinkConfig {
    name: string
    params: {
        locale: string
        [key: string]: string
    }
}

export interface AggregationEvent {
    layer: WMSLayer,
    feature: string
}

export interface TimeseriesEvent {
    layer: WMSLayer,
    layerTime: string,
    connected_timeseries_layers: WMSLayer[],
    connected_single_value_layers: WMSLayer[],
    coordinate: Coordinate
    positionWgs84: Coordinate
}

export interface StaColor {
    color: string,
    position: number,
    upper_limit: number,
    radius: number
}